@@ -128,7 +128,7 @@ static void cb_candidate_gathering_done(NiceAgent *agent, guint stream_id, gpoin
 
 static void cb_component_state_changed (NiceAgent *agent, guint stream_id, guint component_id, guint state, gpointer data)
 {
-  g_debug ("test-dribble:%s: %p", __func__, data);
+  g_debug ("test-dribble:%s: %p", G_STRFUNC, data);
 
   if (GPOINTER_TO_UINT (data) == 1)
     global_lagent_state = state;
@@ -150,14 +150,6 @@ static void cb_component_state_changed (NiceAgent *agent, guint stream_id, guint
     return;
   }
 
-#if 0
-  /* signal status via a global variable */
-  if (global_components_failed == global_components_failed_exit) {
-    g_main_loop_quit (global_mainloop); 
-    return;
-  }
-#endif
-
   /* XXX: dear compiler, these are for you: */
   (void)agent; (void)stream_id; (void)data; (void)component_id;
 }
@@ -165,7 +157,7 @@ static void cb_component_state_changed (NiceAgent *agent, guint stream_id, guint
 static void cb_new_selected_pair(NiceAgent *agent, guint stream_id, guint component_id, 
 				 gchar *lfoundation, gchar* rfoundation, gpointer data)
 {
-  g_debug ("test-dribble:%s: %p", __func__, data);
+  g_debug ("test-dribble:%s: %p", G_STRFUNC, data);
 
   if (GPOINTER_TO_UINT (data) == 1)
     ++global_lagent_cands;
@@ -179,7 +171,7 @@ static void cb_new_selected_pair(NiceAgent *agent, guint stream_id, guint compon
 static void cb_new_candidate(NiceAgent *agent, guint stream_id, guint component_id, 
 			     gchar *foundation, gpointer data)
 {
-  g_debug ("test-dribble:%s: %p", __func__, data);
+  g_debug ("test-dribble:%s: %p", G_STRFUNC, data);
 
   /* XXX: dear compiler, these are for you: */
   (void)agent; (void)stream_id; (void)data; (void)component_id; (void)foundation;
@@ -187,7 +179,7 @@ static void cb_new_candidate(NiceAgent *agent, guint stream_id, guint component_
 
 static void cb_initial_binding_request_received(NiceAgent *agent, guint stream_id, gpointer data)
 {
-  g_debug ("test-dribble:%s: %p", __func__, data);
+  g_debug ("test-dribble:%s: %p", G_STRFUNC, data);
 
   if (GPOINTER_TO_UINT (data) == 1)
     global_lagent_ibr_received = TRUE;
@@ -207,8 +199,17 @@ int main (void)
   GSList *cands, *i;
   guint ls_id, rs_id;
 
+#ifdef G_OS_WIN32
+  WSADATA w;
+
+  WSAStartup(0x0202, &w);
+#endif
+
   g_type_init ();
+#if !GLIB_CHECK_VERSION(2,31,8)
   g_thread_init (NULL);
+#endif
+
   global_mainloop = g_main_loop_new (NULL, FALSE);
 
   /* step: create the agents L and R */
@@ -394,5 +395,8 @@ int main (void)
 
   g_source_remove (timer_id);
 
+#ifdef G_OS_WIN32
+  WSACleanup();
+#endif
   return 0;
 }
